% Main Fuction %
function pkm1
 
%------- Figure settings 
figure('KeyPressFcn',@key_callback);
 
%------- Figure Settings
%view(125,25); 
axis square
axis([-200, 200, -200, 200]);
xlabel('x-axis'); ylabel('y-axis');
grid on;
 
% Keyboard Interrrupt %
function key_callback(obj,event)
cla;                                % Erase the current figure %
persistent pos ang th pos_p ang_p
 
D2R=pi/180; R2D=180/pi;
dpos=5; dth=5*D2R;

%----- ra, rb, L1, L2 ----%
param=[150,35,100,100];
 
if isempty(pos), pos=[0,0]'; pos_p=[0,0]; end
if isempty(ang), ang=0; ang_p=0; end
if isempty(th), th=D2R*[-144.9004,-24.9004,95.0996]'; end
 
key=event.Character;
switch key
%{    
    case '1', th(1,1)=th(1,1)+dth; index=1;        % +th1 Movement %
    case 'q', th(1,1)=th(1,1)-dth; index=1;        % -th1 Movement %
    case '2', th(2,1)=th(2,1)+dth; index=1;        % +th2 Movement %
    case 'w', th(2,1)=th(2,1)-dth; index=1;        % -th2 Movement %
    case '3', th(3,1)=th(3,1)+dth; index=1;        % +th3 Movement %
    case 'e', th(3,1)=th(3,1)-dth; index=1;        % -th3 Movement %                
 %}
    case 'a', pos(1,1)=pos(1,1)+dpos; index=2;      % +X Movement %
    case 'z', pos(1,1)=pos(1,1)-dpos; index=2;      % -X Movement %
    case 's', pos(2,1)=pos(2,1)+dpos; index=2;      % +Y Movement %
    case 'x', pos(2,1)=pos(2,1)-dpos; index=2;      % -Y Movement %
    case 'd', ang=ang+dth; index=2;                 % +ANG Movement %
    case 'c', ang=ang-dth; index=2;                 % -ANG Movement %        
        
    case 'p', pos=[0,0]'; ang=0; th=D2R*[-144.9004,-24.9004,95.0996]'; index=2;   % Original Pos & Orien %        
    otherwise
        disp('Not Proper Key'); index=1;
end
 
% Draw the Fixed Frame %
sc_f=50; draw_frame(sc_f);         % Fixed Frame %
 
switch index
    case 1, % Forward Kinematics %
%        T03=forward_kinematics(th, param);
%        pos=T03(1:2,4); ang=atan2(T03(2,2),T03(1,1));
    case 2, % Inverse Kinematics %
        [th, w_index]=inverse_kinematics(pos, ang, param);
        if w_index==-1, pos=pos_p; ang=ang_p; end 
end
        
% Draw a 3-DOF Planar Robot %
draw_pkm(pos, ang, param);

clc;
disp('px[mm], py[mm], ang[deg]');
disp([pos(1,1), pos(2,1), ang*R2D]);
disp('th1[deg], th2[deg], th3[deg]');
disp([th(1,1), th(2,1), th(3,1)]*R2D);

[Jm]=Jacobian_Matrix(pos, ang, param);
disp('Jacobian Matrix');
disp(Jm);
tau=inv(Jm)*[0,10,0]';
disp('Torque [Nmm]');
disp(tau');

pos_p=pos; ang_p=ang;
 
%-------- Inverse Kinematics
function [th, w_index]=inverse_kinematics(pos, ang, param)
w_index=1;  % In workspace %
ra=param(1); rb=param(2); L1=param(3); L2=param(4); 

% Vertex Points %
A=ra*[0,-sqrt(3)/2, +sqrt(3)/2; 1, -1/2, -1/2];
B=rb*[0,-sqrt(3)/2, +sqrt(3)/2; 1, -1/2, -1/2];

% (2)
th=zeros(3,1);
R=[cos(ang), -sin(ang); sin(ang), cos(ang)];
for i=1:3
    d=pos+R*B(:,i)-A(:,i);
    e1=-2*L1*d(2,1); e2=-2*L1*d(1,1);
    e3=d(1,1)^2+d(2,1)^2+L1^2-L2^2;
    temp=e1^2+e2^2-e3^2;
    if temp>=0
        th(i,1)=2*atan((-e1-sqrt(temp))/(e3-e2));
    else
        th(i,1)=0;
        w_index=-1;
        disp('Out of Workspace');
    end
end

function [Jm]=Jacobian_Matrix(pos, ang, param)
ra=param(1); rb=param(2); L1=param(3); L2=param(4); 
% Vertex Points %
A=ra*[0,-sqrt(3)/2, +sqrt(3)/2; 1, -1/2, -1/2];
B=rb*[0,-sqrt(3)/2, +sqrt(3)/2; 1, -1/2, -1/2];
R=[cos(ang), -sin(ang); sin(ang), cos(ang)];

[th]=inverse_kinematics(pos, ang, param);

Jm=zeros(3,3);
for i=1:3
    r=R*B(:,i);
    Lv=(R*B(:,i)+pos)-(A(:,i)+L1*[cos(th(i,1));sin(th(i,1))]);
    s=Lv/norm(Lv);
    sr=[s; (r(1,1)*s(2,1)-r(2,1)*s(1,1))];
    S1=[(A(2,i)-pos(2,1)),-(A(1,i)-pos(1,1)),1]';
    Jm(:,i)=sr/dot(sr,S1);
end
  
% Draw Primitives %
%-------- Draw a 3-DOF Planar Robot
function draw_pkm(pos, ang, param)
ra=param(1); rb=param(2); L1=param(3);
% Vertex Points %
A=ra*[0,-sqrt(3)/2, +sqrt(3)/2; 1, -1/2, -1/2]; pa=A;
B=rb*[0,-sqrt(3)/2, +sqrt(3)/2; 1, -1/2, -1/2];

[th]=inverse_kinematics(pos, ang, param);

% Draw Fixed Frame %
lw=2; clr=[0.5,0.1,0.1];
draw_line(pa(:,1),pa(:,2),lw,clr);
draw_line(pa(:,2),pa(:,3),lw,clr);
draw_line(pa(:,3),pa(:,1),lw,clr);

% Draw Moving Frame %
R=[cos(ang), -sin(ang); sin(ang), cos(ang)];
pb=zeros(2,3);
lw=2; clr=[0.5,0.1,0.1];
for i=1:3
    pb(:,i)=R*B(:,i)+pos;
end    
draw_line(pb(:,1),pb(:,2),lw,clr);
draw_line(pb(:,2),pb(:,3),lw,clr);
draw_line(pb(:,3),pb(:,1),lw,clr);

% Draw Arms %
lw=2; clr=[0.5,0.1,0.1];
pm=zeros(2,3);
for i=1:3
    pm(:,i)=A(:,i)+L1*[cos(th(i,1));sin(th(i,1))];
    draw_line(A(:,i),pm(:,i),lw,clr);
end

% Draw Connecting Links %
lw=2; clr=[0.5,0.1,0.1];
for i=1:3
    draw_line(pm(:,i),pb(:,i),lw,clr);
end

% Draw the Moving Frame & Box %
sc=20;
p1=pos; p2=pos+R*[sc;0]; lw=2; clr=[1,0,0];
draw_line(p1,p2,lw,clr);
p1=pos; p2=pos+R*[0;sc]; lw=2; clr=[0,1,0];
draw_line(p1,p2,lw,clr);

%-------- Draw a Frame
function draw_frame(sc)
p1=[0,0]'; p2=[sc,0]'; lw=2; clr=[1,0,0];
draw_line(p1,p2,lw,clr);
p1=[0,0]'; p2=[0,sc]'; lw=2; clr=[0,1,0];
draw_line(p1,p2,lw,clr);
 
%------- Draw a Line
function draw_line(p1, p2, lw, clr)
line([p1(1,1),p2(1,1)],[p1(2,1),p2(2,1)],'LineWidth',lw,'Color',clr);

% Homogeneous Transformation Matrix %
%------- Rotation matrix about the X-axis 
function [T]=Rotx(th)
cx=cos(th); sx=sin(th);
T=[1,0,0,0; 0,cx,-sx,0; 0,sx,cx,0; 0,0,0,1];
 
%------- Rotation matrix about the Y-axis 
function [T]=Roty(th)
cy=cos(th); sy=sin(th);
T=[cy,0,sy,0; 0,1,0,0; -sy,0,cy,0; 0,0,0,1];
 
%------- Rotation matrix about the Z-axis 
function [T]=Rotz(th)
cz=cos(th); sz=sin(th);
T=[cz,-sz,0,0; sz,cz,0,0; 0,0,1,0; 0,0,0,1];
 
%-------- Translation along the x, y, z axes 
function [T]=Trans(p)
T=[1,0,0,p(1,1); 0,1,0,p(2,1); 0,0,1,p(3,1); 0,0,0,1];
