% 3-DOF Serial Robot
% Trajectory Planning in the Cartesian Space

function []=dyn_3dof()
clear all;
global MAX_CH
MAX_CH=3;
D2R=pi/180;

a1=100; a2=70; a3=50;
phi=90*D2R;
x0=[a1,a2+a3]';
xf=[-a1,a2+a3]';
tf=1; dt=0.001;

x=zeros(2,1); x_dot=zeros(2,1); x_ddot=zeros(2,1);
[c] = coeff_3rd(x0, zeros(MAX_CH,1), xf, zeros(MAX_CH,1), tf);

n=1;
for t=0:dt:tf   
    for ch=1:2
        x(ch,1)=c(ch,1)+c(ch,2)*t+c(ch,3)*t^2+c(ch,4)*t^3;
        x_dot(ch,1)=c(ch,2)+2*c(ch,3)*t+3*c(ch,4)*t^2;
        x_ddot(ch,1)=2*c(ch,3)+6*c(ch,4)*t;
    end
    [th]=inverse_kinematics(x, phi);
    [th_dot, th_ddot]=derivatives(th, dt);
    
    % Required Torques %
    [T]=dynamics(th_ddot, th_dot, th);
    x_data(n,1)=t;
    y_data1(n,:)=[x(1,1), x_dot(1,1), x_ddot(1,1)];
    y_data2(n,:)=[th(1,1), th_dot(1,1), th_ddot(1,1)];
    y_data3(n,:)=[th(2,1), th_dot(2,1), th_ddot(2,1)];
    y_data4(n,:)=[th(3,1), th_dot(3,1), th_ddot(3,1)];
    %y_data5(n,:)=T;
    y_data5(n,:)=T;
    n=n+1;
end
 
figure(2)
subplot(2,1,1), plot(x_data, y_data1);
xlabel('time [sec]'); ylabel('pos[mm], vel[mm/s], acc[mm/s^2]'); grid on;
subplot(2,1,2), plot(x_data, y_data5);
xlabel('time [sec]'); ylabel('Torque [Nmm]'); grid on;

function [v, a]=derivatives(x, dt)
global MAX_CH
persistent xp vp

if isempty(xp), xp=x; end
if isempty(vp), vp=zeros(MAX_CH,1); end

v=(x-xp)/dt;     
a=(v-vp)/dt;   

% Back substitutions %
xp=x; vp=v;

function [th, w_index]  = inverse_kinematics(x, phi)

w_index=1;  % In workspace %
a1=100; a2=70; a3=50;

qx=x(1,1); qy=x(2,1);

% (1) Calculate th2 %
px=qx-a3*cos(phi); py=qy-a3*sin(phi);
kapha=(px^2+py^2-a1^2-a2^2)/(2*a1*a2);
if abs(kapha)>1
    w_index=-1;  % Out of Workspace %
    kapha=1;
end
th2=+acos(kapha); % Elbow Down %
%th2=-acos(kapha); % Elbow Up %
 
% (2) Calculate th1 %
delta=a1^2+a2^2+2*a1*a2*cos(th2);
cth1=(px*(a1+a2*cos(th2))+py*a2*sin(th2))/delta;
sth1=(-px*a2*sin(th2)+py*(a1+a2*cos(th2)))/delta;
th1=atan2(sth1, cth1);
if th1<0, th1=th1+2*pi; end % 0<th<360 %

% (3) Calcuate th3 %
th3=phi-(th1+th2);

th=[th1, th2, th3]';

%-------------- Only Gravity Force ----------------%
function [T]=dynamics(th_ddot, th_dot, th)
% Robot Parameters %
m1=0.363; a1=100; ac1=87.74; Ic1=313.923;
m2=0.292; a2=70; ac2=62.65; Ic2=138.353;
m3=0.050; a3=50; ac3=18.60; Ic3=24.587;
g=9806.65;

% State Variables %
th1=th(1,1); th2=th(2,1); th3=th(3,1);

G(1,1)=g*(m1*ac1*cos(th1)+m2*(a1*cos(th1)+ac2*cos(th1+th2))...
    +m3*(a1*cos(th1)+a2*cos(th1+th2))+ac3*cos(th1+th2+th3));
G(2,1)=m2*g*ac2*cos(th1+th2)+m3*g*(a2*cos(th1+th2)+ac3*cos(th1+th2+th3));
G(3,1)=m3*g*ac3*cos(th1+th2+th3);
 
T=0.001*G;

%---------------------- Trajectory Functions
function [c] = coeff_3rd(sp, sv, ep, ev, tsec)
MAX_CH=2;
c=zeros(MAX_CH,4);
for ch=1:MAX_CH
    c(ch,1)=sp(ch,1);
    c(ch,2)=sv(ch,1);
    c(ch,3)=(3.0*(ep(ch,1)-sp(ch,1))-(2.0*sv(ch,1)+ev(ch,1))*tsec)/(tsec*tsec);
    c(ch,4)=(-2.0*(ep(ch,1)-sp(ch,1))+(sv(ch,1)+ev(ch,1))*tsec)/(tsec*tsec*tsec);
end
