%% Main Fuction %%
function []=sixbar1()
clc;

% Constants %
D2R=pi/180; R2D=180/pi;

%{
% Link Lengths %
pitch=7.5;
x1=2*pitch; x2=7*pitch; x3=5*pitch; nx3=8*pitch;
Z1=x2; Z2=x1; Z3=x3; Z4=x3; Z7=x3;
Z5=nx3; Z6=nx3; Z8=nx3-Z7;
%}

n=1;
for th2=0*D2R: 5*D2R: 360*D2R
    
    [pmat]=Left_Mechanism(th2);

    % Draw a 3-DOF Planar Robot %
    %------- Figure Settings
    clf;
    figure(1);
    axis([-150, 150, -150, 150]);
    xlabel('x-axis'); ylabel('y-axis');
    grid on;
    draw_mechanism(pmat);
    
    fpoint(:,n)=pmat(:,7);
    xdata(n)=th2/D2R;
    ydata(:,n)=pmat(:,7);
        
    n=n+1;
    pause(0.01);
%{    
    disp('C34:'); disp(c34');
    disp('C78:'); disp(c78');
    disp('C56:'); disp(c56');
    disp('C910:'); disp(c910');
    disp('C1112:'); disp(c1112');
%}
end

% Draw the trace of the foot point %
draw_fpoint(fpoint);

% Results %
figure(2);
plot(xdata, ydata(1,:),'k-', xdata, ydata(2,:),'k:');
xlabel('theta2 [deg]');
ylabel('X & Y Positions [mm]');
grid on;

function [pmat]=Left_Mechanism(th2)

% Link Lengths %
pitch=7.5;
x1=2*pitch; x2=6*pitch; x3=5*pitch; nx3=8*pitch;
%r1=52.5/108; x1=38*r1; x2=108*r1; x3=83*r1; nx3=2.92*x3; %nx3/pitch
Z1=x2; Z2=x1; Z3=x3; Z4=x3; Z7=x3;
Z5=nx3; Z6=nx3; Z8=nx3-Z7;

% Step0: Inputs %
c12=[0,0]';
c14=Z1*[-1,0]';
c23=Z2*[cos(th2),sin(th2)]';

% Step1: Find C34 from C23 & C14 %
[c34]=inter_point(c14,Z4,c23,Z3,-1);
% Step2: Find C67 from C23 & C34 %
[c67]=external_point(c23,c34,Z6);
% Step3: Find C57 from C14 & C67 %
[c57]=inter_point(c14,Z5,c67,Z7,-1);
% Step4: Find C8 from C57 & C67 %
[c8]=external_point(c57,c67,Z8);
pmat=[c12,c23,c34,c14,c57,c67,c8];

% Find the intersection points of two circles %
function [pout]=inter_point(c1,r1,c2,r2, index)

th=atan2(c2(2,1)-c1(2,1),c2(1,1)-c1(1,1));
d=norm(c2-c1);
if d>1.e-8
    px1=(d^2+r1^2-r2^2)/(2*d);
    py1=index*sqrt(r1^2-px1^2);
else
    px1=0;
    py1=0;
end
pout=[cos(th)*px1-sin(th)*py1+c1(1,1);...
    sin(th)*px1+cos(th)*py1+c1(2,1)];

% Find the intersection points of two circles %
function [pout]=external_point(c1,c2, r)
uv=(c2-c1)/norm(c2-c1);
pout=r*uv+c2;

%% Draw Primitives %%
%-------- Draw a Mechanism
function draw_mechanism(pm)
%pmat=[c12,c23,c34,c14,c57,c67,c8];

sc_f=50; draw_frame(eye(4,4), sc_f);         % Fixed Frame %

lw=2; clr=[0,0,1];
% Draw the Z1 %
draw_line(pm(:,1),pm(:,4),lw,clr);
% Draw the Z2 %
draw_line(pm(:,1),pm(:,2),lw,clr);
% Draw the Z3 +Z6 %
draw_line(pm(:,2),pm(:,6),lw,clr);

clr=[0,1,0.5];
% Draw the Z4 %
draw_line(pm(:,3),pm(:,4),lw,clr);

clr=[0,1,1];
% Draw the Z5 %
draw_line(pm(:,4),pm(:,5),lw,clr);
% Draw the Z7+Z8 %
draw_line(pm(:,5),pm(:,7),lw,clr);

function draw_fpoint(fpoint)

[m,n]=size(fpoint);

lw=1; clr=[0,0,0];
for i=1:n-1
    draw_line(fpoint(:,i), fpoint(:,i+1),lw,clr);
end
draw_line(fpoint(:,n), fpoint(:,1),lw,clr);
    

%-------- Draw a Frame
function draw_frame(T, sc)
p1=T*[0,0,0,1]'; p2=T*[sc,0,0,1]'; lw=2; clr=[1,0,0];
draw_line(p1(1:2,1),p2(1:2,1),lw,clr);
p1=T*[0,0,0,1]'; p2=T*[0,sc,0,1]'; lw=2; clr=[0,1,0];
draw_line(p1(1:2,1),p2(1:2,1),lw,clr);
p1=T*[0,0,0,1]'; p2=T*[0,0,sc,1]'; lw=2; clr=[0,0,1];
draw_line(p1(1:2,1),p2(1:2,1),lw,clr);
 
%------- Draw a Line
function draw_line(p1, p2, lw, clr)
line([p1(1,1),p2(1,1)],[p1(2,1),p2(2,1)],'LineWidth',lw,'Color',clr);

